package java.lang;

import java.security.SecurityPermission;
import sun.reflect.CallerSensitive;
import java.security.Security;
import java.security.PrivilegedAction;
import java.util.StringTokenizer;
import java.util.PropertyPermission;
import java.net.InetAddress;
import java.net.SocketPermission;
import java.io.FileDescriptor;
import java.io.FilePermission;
import java.io.File;
import java.security.AccessControlContext;
import java.security.Permission;
import java.security.AccessController;
import sun.security.util.SecurityConstants;

public class SecurityManager
{
    @Deprecated
    protected boolean inCheck;
    private boolean initialized;
    private static ThreadGroup rootGroup;
    private static boolean packageAccessValid;
    private static String[] packageAccess;
    private static final Object packageAccessLock;
    private static boolean packageDefinitionValid;
    private static String[] packageDefinition;
    private static final Object packageDefinitionLock;
    
    private boolean hasAllPermission() {
        try {
            this.checkPermission(SecurityConstants.ALL_PERMISSION);
            return true;
        }
        catch (SecurityException ex) {
            return false;
        }
    }
    
    @Deprecated
    public boolean getInCheck() {
        return this.inCheck;
    }
    
    public SecurityManager() {
        this.initialized = false;
        synchronized (SecurityManager.class) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(new RuntimePermission("createSecurityManager"));
            }
            this.initialized = true;
        }
    }
    
    protected native Class[] getClassContext();
    
    @Deprecated
    protected ClassLoader currentClassLoader() {
        ClassLoader currentClassLoader0 = this.currentClassLoader0();
        if (currentClassLoader0 != null && this.hasAllPermission()) {
            currentClassLoader0 = null;
            return currentClassLoader0;
        }
        return currentClassLoader0;
    }
    
    private native ClassLoader currentClassLoader0();
    
    @Deprecated
    protected Class<?> currentLoadedClass() {
        Class<Object> currentLoadedClass0 = this.currentLoadedClass0();
        if (currentLoadedClass0 != null && this.hasAllPermission()) {
            currentLoadedClass0 = null;
            return currentLoadedClass0;
        }
        return currentLoadedClass0;
    }
    
    @Deprecated
    protected native int classDepth(String p0);
    
    @Deprecated
    protected int classLoaderDepth() {
        int classLoaderDepth0 = this.classLoaderDepth0();
        if (classLoaderDepth0 != -1) {
            if (this.hasAllPermission()) {
                classLoaderDepth0 = -1;
            }
            else {
                --classLoaderDepth0;
            }
        }
        return classLoaderDepth0;
    }
    
    private native int classLoaderDepth0();
    
    @Deprecated
    protected boolean inClass(String s) {
        return this.classDepth(s) >= 0;
    }
    
    @Deprecated
    protected boolean inClassLoader() {
        return this.currentClassLoader() != null;
    }
    
    public Object getSecurityContext() {
        return AccessController.getContext();
    }
    
    public void checkPermission(Permission perm) {
        AccessController.checkPermission(perm);
    }
    
    public void checkPermission(Permission perm, Object o) {
        if ((o instanceof AccessControlContext)) {
            ((AccessControlContext)o).checkPermission(perm);
            return;
        }
        throw new SecurityException();
    }
    
    public void checkCreateClassLoader() {
        this.checkPermission(SecurityConstants.CREATE_CLASSLOADER_PERMISSION);
    }
    
    private static ThreadGroup getRootGroup() {
        ThreadGroup threadGroup;
        for (threadGroup = Thread.currentThread().getThreadGroup(); threadGroup.getParent() != null; threadGroup = threadGroup.getParent()) {}
        return threadGroup;
    }
    
    public void checkAccess(Thread thread) {
        if (thread == null) {
            throw new NullPointerException("thread can't be null");
        }
        if (thread.getThreadGroup() == SecurityManager.rootGroup) {
            this.checkPermission(SecurityConstants.MODIFY_THREAD_PERMISSION);
        }
        return;
    }
    
    public void checkAccess(ThreadGroup threadGroup) {
        if (threadGroup == null) {
            throw new NullPointerException("thread group can't be null");
        }
        if (threadGroup == SecurityManager.rootGroup) {
            this.checkPermission(SecurityConstants.MODIFY_THREADGROUP_PERMISSION);
        }
        return;
    }
    
    public void checkExit(int n) {
        this.checkPermission(new RuntimePermission("exitVM." + n));
    }
    
    public void checkExec(String s) {
        if (new File(s).isAbsolute()) {
            this.checkPermission(new FilePermission(s, "execute"));
        }
        else {
            this.checkPermission(new FilePermission("<<ALL FILES>>", "execute"));
        }
    }
    
    public void checkLink(String s) {
        if (s == null) {
            throw new NullPointerException("library can't be null");
        }
        this.checkPermission(new RuntimePermission("loadLibrary." + s));
        return;
    }
    
    public void checkRead(FileDescriptor fileDescriptor) {
        if (fileDescriptor == null) {
            throw new NullPointerException("file descriptor can't be null");
        }
        this.checkPermission(new RuntimePermission("readFileDescriptor"));
        return;
    }
    
    public void checkRead(String path) {
        this.checkPermission(new FilePermission(path, "read"));
    }
    
    public void checkRead(String path, Object o) {
        this.checkPermission(new FilePermission(path, "read"), o);
    }
    
    public void checkWrite(FileDescriptor fileDescriptor) {
        if (fileDescriptor == null) {
            throw new NullPointerException("file descriptor can't be null");
        }
        this.checkPermission(new RuntimePermission("writeFileDescriptor"));
        return;
    }
    
    public void checkWrite(String path) {
        this.checkPermission(new FilePermission(path, "write"));
    }
    
    public void checkDelete(String path) {
        this.checkPermission(new FilePermission(path, "delete"));
    }
    
    public void checkConnect(String string, int n) {
        if (string == null) {
            throw new NullPointerException("host can't be null");
        }
        if (!string.startsWith("[") && string.indexOf(58) != -1) {
            string = "[" + string + "]";
        }
        if (n == -1) {
            this.checkPermission(new SocketPermission(string, "resolve"));
        }
        else {
            this.checkPermission(new SocketPermission(string + ":" + n, "connect"));
        }
        return;
    }
    
    public void checkConnect(String string, int n, Object o) {
        if (string == null) {
            throw new NullPointerException("host can't be null");
        }
        if (!string.startsWith("[") && string.indexOf(58) != -1) {
            string = "[" + string + "]";
        }
        if (n == -1) {
            this.checkPermission(new SocketPermission(string, "resolve"), o);
        }
        else {
            this.checkPermission(new SocketPermission(string + ":" + n, "connect"), o);
        }
        return;
    }
    
    public void checkListen(int n) {
        this.checkPermission(new SocketPermission("localhost:" + n, "listen"));
    }
    
    public void checkAccept(String string, int n) {
        if (string == null) {
            throw new NullPointerException("host can't be null");
        }
        if (!string.startsWith("[") && string.indexOf(58) != -1) {
            string = "[" + string + "]";
        }
        this.checkPermission(new SocketPermission(string + ":" + n, "accept"));
        return;
    }
    
    public void checkMulticast(InetAddress inetAddress) {
        String host = inetAddress.getHostAddress();
        if (!host.startsWith("[") && host.indexOf(58) != -1) {
            host = "[" + host + "]";
        }
        this.checkPermission(new SocketPermission(host, "connect,accept"));
    }
    
    @Deprecated
    public void checkMulticast(InetAddress inetAddress, byte b) {
        String host = inetAddress.getHostAddress();
        if (!host.startsWith("[") && host.indexOf(58) != -1) {
            host = "[" + host + "]";
        }
        this.checkPermission(new SocketPermission(host, "connect,accept"));
    }
    
    public void checkPropertiesAccess() {
        this.checkPermission(new PropertyPermission("*", "read,write"));
    }
    
    public void checkPropertyAccess(String name) {
        this.checkPermission(new PropertyPermission(name, "read"));
    }
    
    @Deprecated
    public boolean checkTopLevelWindow(Object o) {
        if (o == null) {
            throw new NullPointerException("window can't be null");
        }
        Permission permission = SecurityConstants.AWT.TOPLEVEL_WINDOW_PERMISSION;
        if (permission == null) {
            permission = SecurityConstants.ALL_PERMISSION;
        }
        try {
            this.checkPermission(permission);
            return true;
        }
        catch (SecurityException ex) {
            return false;
        }
    }
    
    public void checkPrintJobAccess() {
        this.checkPermission(new RuntimePermission("queuePrintJob"));
    }
    
    @Deprecated
    public void checkSystemClipboardAccess() {
        Permission permission = SecurityConstants.AWT.ACCESS_CLIPBOARD_PERMISSION;
        if (permission == null) {
            permission = SecurityConstants.ALL_PERMISSION;
        }
        this.checkPermission(permission);
    }
    
    @Deprecated
    public void checkAwtEventQueueAccess() {
        Permission permission = SecurityConstants.AWT.CHECK_AWT_EVENTQUEUE_PERMISSION;
        if (permission == null) {
            permission = SecurityConstants.ALL_PERMISSION;
        }
        this.checkPermission(permission);
    }
    
    private static String[] getPackages(String str) {
        String[] array = null;
        if (str != null && !str.equals("")) {
            StringTokenizer stringTokenizer = new StringTokenizer(str, ",");
            int countTokens = stringTokenizer.countTokens();
            if (countTokens > 0) {
                array = new String[countTokens];
                int n = 0;
                while (stringTokenizer.hasMoreElements()) {
                    array[n++] = stringTokenizer.nextToken().trim();
                }
            }
        }
        if (array == null) {
            array = new String[0];
            return array;
        }
        return array;
    }
    
    public void checkPackageAccess(String s) {
        if (s == null) {
            throw new NullPointerException("package name can't be null");
        }
        String[] packageAccess;
        synchronized (SecurityManager.packageAccessLock) {
            if (!SecurityManager.packageAccessValid) {
                SecurityManager.packageAccess = getPackages((String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                    @Override
                    public String run() {
                        return Security.getProperty("package.access");
                    }
                }));
                SecurityManager.packageAccessValid = true;
            }
            packageAccess = SecurityManager.packageAccess;
        }
        int i = 0;
        while (i < packageAccess.length) {
            if (s.startsWith(packageAccess[i]) || packageAccess[i].equals(s + ".")) {
                this.checkPermission(new RuntimePermission("accessClassInPackage." + s));
                break;
            }
            ++i;
        }
        return;
    }
    
    public void checkPackageDefinition(String s) {
        if (s == null) {
            throw new NullPointerException("package name can't be null");
        }
        String[] packageDefinition;
        synchronized (SecurityManager.packageDefinitionLock) {
            if (!SecurityManager.packageDefinitionValid) {
                SecurityManager.packageDefinition = getPackages((String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                    @Override
                    public String run() {
                        return Security.getProperty("package.definition");
                    }
                }));
                SecurityManager.packageDefinitionValid = true;
            }
            packageDefinition = SecurityManager.packageDefinition;
        }
        int i = 0;
        while (i < packageDefinition.length) {
            if (s.startsWith(packageDefinition[i]) || packageDefinition[i].equals(s + ".")) {
                this.checkPermission(new RuntimePermission("defineClassInPackage." + s));
                break;
            }
            ++i;
        }
        return;
    }
    
    public void checkSetFactory() {
        this.checkPermission(new RuntimePermission("setFactory"));
    }
    
    @Deprecated
    @CallerSensitive
    public void checkMemberAccess(Class<?> clazz, int n) {
        if (clazz == null) {
            throw new NullPointerException("class can't be null");
        }
        if (n != 0) {
            Class[] classContext = this.getClassContext();
            if (classContext.length < 4 || classContext[3].getClassLoader() != clazz.getClassLoader()) {
                this.checkPermission(SecurityConstants.CHECK_MEMBER_ACCESS_PERMISSION);
            }
        }
        return;
    }
    
    public void checkSecurityAccess(String name) {
        this.checkPermission(new SecurityPermission(name));
    }
    
    private native Class<?> currentLoadedClass0();
    
    public ThreadGroup getThreadGroup() {
        return Thread.currentThread().getThreadGroup();
    }
    
    static {
        SecurityManager.rootGroup = getRootGroup();
        SecurityManager.packageAccessValid = false;
        packageAccessLock = new Object();
        SecurityManager.packageDefinitionValid = false;
        packageDefinitionLock = new Object();
    }
}
